/*
 * Example program.  Randomly fill up the screen with numbers until
 * it all turns to asterisks and occasionally show a frame to show
 * the status.  A child process is started which doesn't do much.
 */

#include <unistd.h>
#include <stdlib.h>

#include <dpy.h>


static void startChild(void);


int
main(int argc, const char ** argv)
{
	int row;
	int col;
	int ch;
	int rows;
	int cols;
	int leftCount;
	int checkCount = 0;
	int changeCount = 0;
	char * saveData = 0;

	if (!dpyInit(0, "-e"))
		return 1;

	startChild();

	dpyGetSize(&rows, &cols);

	leftCount = rows * cols * 10;

	dpyHome();
	dpyUpdate();

	dpySetDefaultFlags(DPY_LEAVE_CURSOR);

	sleep(1);

	do
	{
		row = random() % rows;
		col = random() % cols;

		ch = dpyGetChar(row, col);

		changeCount++;
		leftCount--;

		if (ch == ' ')
			ch = '1';
		else if (ch == '9')
			ch = '*';
		else if (ch != '*')
			ch++;
		else
		{
			leftCount++;
			changeCount--;
		}

		dpyPlaceChar(row, col, ch);

		dpyUpdate();

		if ((++checkCount % 1000) == 0)
		{
			if (!dpySave(&saveData))
				continue;

			sleep(1);

			dpyFrame(rows / 3, (rows / 3) + 6,
				cols / 3, (cols / 3) + 36);

			dpyPrintf("\n"
				"    Total checks: %d\n"
				"    Changes this update: %d\n"
				"    Changes left to make: %d\n",
				checkCount, changeCount, leftCount);

			dpyClearWindow();
			dpyHome();
			dpySetFlags(-1, 0);

			dpyUpdate();

			sleep(1);

			dpyRestore(saveData);

			dpyWindow(0, -1, 0, -1);
			dpyUpdate();

			changeCount = 0;
		}
	}
	while ((leftCount > 0) || (changeCount > 0) || (checkCount % 1000));

	sleep(1);

	dpyClose();

	return 0;
}


/*
 * Demo routine to demonstrate creating a child process which using dpy.
 * It doesn't do anything useful.
 */
static void
startChild(void)
{
	pid_t pid = fork();

	if (pid != 0)
		return;

	dpyChildClose();

	while (1)
		sleep(10);
}


